#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-11-18
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93

STATE_INIT = 100
STATE_TIMING = 101
STATE_RUNNING = 102
STATE_SCHEDULE_TIMEOUT = 103
STATE_RUNNING_TIMEOUT = 104
STATE_DOWNLOAD_FAIL = 105
STATE_SUCCESS = 0
STATE_FAIL = 1

ALL_STATES = set([STATE_INIT, STATE_TIMING, STATE_RUNNING, STATE_SCHEDULE_TIMEOUT,
                  STATE_RUNNING_TIMEOUT, STATE_DOWNLOAD_FAIL, STATE_SUCCESS, STATE_FAIL])

#INIT_STATE = 0
#TIMING_STATE = 1
#SUCCESS_STATE = 2
#RUNNING_STATE = 3
#SCHEDULE_TIMEOUT_STATE = 4
#RUNNING_TIMEOUT_STATE = 5
#FAIL = 6
